package cx.lehmann.gemini.gemini_proxy;

import java.util.Date;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import io.vertx.core.AbstractVerticle;
import io.vertx.core.Promise;
import io.vertx.core.net.NetClient;
import io.vertx.core.net.NetClientOptions;
import io.vertx.core.net.NetServerOptions;
import io.vertx.core.net.NetSocket;
import io.vertx.core.net.PemKeyCertOptions;
import io.vertx.core.net.ProxyOptions;
import io.vertx.core.net.ProxyType;
import io.vertx.core.streams.Pump;

public class MainVerticle extends AbstractVerticle {

  private final static Logger log = LoggerFactory.getLogger(MainVerticle.class);

  @Override
  public void start(Promise<Void> startPromise) throws Exception {
    NetServerOptions serverOptions=new NetServerOptions();
    NetClientOptions clientOptions=new NetClientOptions();

    ProxyOptions proxyOptions=new ProxyOptions();
    proxyOptions.setHost("localhost")
      .setPort(9150)
      .setType(ProxyType.SOCKS5);

    clientOptions.setProxyOptions(proxyOptions)
        .setTrustAll(true)
        .setSsl(true);

    NetClient client=vertx.createNetClient(clientOptions);

    vertx.exceptionHandler(ex -> {ex.printStackTrace();});

    String certPath="c:/temp/cert.pem";
//    String certPath="/home/lehmann/gemini-chat/cert.pem";

    serverOptions.setPemKeyCertOptions(new PemKeyCertOptions()
        .setCertPath(certPath)
        .setKeyPath(certPath))
        .setSsl(true);

    vertx.createNetServer(serverOptions).connectHandler(conn -> {
      conn.handler(event -> {
        System.out.println(new Date().toString()+" accepted connection:"+conn.remoteAddress());
        String line=event.toString("UTF-8");
        if(!line.endsWith("\r\n")) {
          conn.write("40 format error\r\n");
          conn.close();
        } else {
          String url=line.substring(0, line.length()-2);

          if(!url.startsWith("gemini://")) {
            conn.write("40 format error\r\n");
            conn.close();
          }
          else {
            String url1=url.substring(9);
            log.info("url:{}",url1);
            int firstSlash=url1.indexOf('/');
            String hostname=url1.substring(0,firstSlash);
            String path=url1.substring(firstSlash);
            client.connect(1965, hostname, res -> {
              if(res.succeeded()) {
                NetSocket clientConn = res.result();
                clientConn.write(url+"\r\n");
                Pump pump=Pump.pump(clientConn, conn).start();
                clientConn.closeHandler(v -> {
                  pump.stop();
                  conn.close();
                });
                conn.closeHandler(v -> {
                  pump.stop();
                  clientConn.close();
                });
              } else {
                log.info("connection failed", res.cause());
                conn.write("40 connection error"+res.cause().getMessage()+"\r\n");
                conn.close();
              }
            });
          }
        }
      }
          );
    }).listen(1965, server -> {
      if (server.succeeded()) {
        startPromise.complete();
        System.out.println("Gemini proxy started on port 1965 at "+new Date().toString());
      } else {
        server.cause().printStackTrace();
        startPromise.fail(server.cause());
      }
    });
  }
}
